// api/cloud.ts
import Taro from '@tarojs/taro'
import { Result, createSuccess, createError } from '@/types/result'

// 请求选项
interface CloudOptions {
  showLoading?: boolean
  loadingText?: string
  showError?: boolean
}

// 默认选项
const defaultOptions: CloudOptions = {
  showLoading: true,
  loadingText: '加载中...',
  showError: true
}

/**
 * 云函数调用封装
 * @param name 云函数名称
 * @param data 请求数据
 * @param options 请求选项
 */
export async function callCloud<T = any>(
  name: string,
  data?: any,
  options?: CloudOptions
): Promise<Result<T>> {
  const opt = { ...defaultOptions, ...options }
  
  // 显示加载中
  if (opt.showLoading) {
    Taro.showLoading({
      title: opt.loadingText || '加载中...'
    })
  }
  
  try {
    // 确保云环境已初始化
    try {
      Taro.cloud.init({
        env: 'cloud1' // 替换为您实际的云环境ID
      })
    } catch (e) {
      // 忽略已初始化的错误
    }

    const result = await Taro.cloud.callFunction({
      name,
      data
    })

    // 隐藏加载中
    if (opt.showLoading) {
      Taro.hideLoading()
    }

    if (result && result.result) {
      // 检查是否已经是标准Result格式
      if (typeof result.result === 'object' && 'success' in result.result) {
        return result.result as Result<T>;
      }
      
      // 返回成功结果
      return createSuccess<T>(result.result as T);
    }
    
    return createError('云函数调用失败: 没有返回结果');
  } catch (error: any) {
    // 隐藏加载中
    if (opt.showLoading) {
      Taro.hideLoading()
    }
    
    // 显示错误
    if (opt.showError) {
      Taro.showToast({
        title: error.message || '请求异常',
        icon: 'none'
      })
    }
    
    console.error(`调用云函数 ${name} 失败:`, error)
    return createError(error.message || `调用云函数 ${name} 失败`);
  }
}

/**
 * 获取云数据库实例
 */
export function getCloudDB() {
  try {
    Taro.cloud.init({
      env: 'cloud1' // 替换为您实际的云环境ID
    })
  } catch (e) {
    // 忽略已初始化的错误
  }
  
  return Taro.cloud.database();
}

/**
 * 云开发API集合
 */
export const cloudApi = {
  call: callCloud,
  db: getCloudDB
};