// stores/user.ts - 管理用户信息、登录状态、角色权限等
import { defineStore } from 'pinia'
import Taro from '@tarojs/taro'
import { userService} from '@/services/user'
import { type UserInfo, UserRole } from '@/types/user'
import { ref, computed } from 'vue'

export const useUserStore = defineStore('user', () => {
  // 基本状态
  const openid = ref('')
  const nickname = ref('')
  const avatar = ref('')
  const role = ref<UserRole | undefined>(UserRole.USER)
  const isRegistered = ref(false)
  const currentRoom = ref<string | null>(null)
  const currentGameId = ref<string | null>(null)
  const loading = ref(false)
  const error = ref<string | null>(null)
  
  // 经验值系统
  const points = ref(0)      // 积分
  const exp = ref(0)         // 经验值
  const level = ref(1)       // 等级
  const nextLevelExp = ref(100) // 下一级所需经验
  
  // 游戏统计数据
  const gameCount = ref(0)   // 参与游戏数量
  const roomCount = ref(0)   // 创建房间数量
  
  // 计算属性
  const userInfo = computed<UserInfo>(() => ({
    openid: openid.value,
    nickname: nickname.value,
    avatar: avatar.value,
    role: role.value
  }))
  
  // 经验值百分比
  const expPercentage = computed(() => {
    return (exp.value / nextLevelExp.value) * 100
  })
  
  // 保存用户状态到本地存储
  function saveUserToStorage() {
    const userData = {
      openid: openid.value,
      nickname: nickname.value,
      avatar: avatar.value,
      role: role.value,
      currentRoom: currentRoom.value,
      isRegistered: isRegistered.value,
      // 经验值系统数据
      points: points.value,
      exp: exp.value,
      level: level.value,
      nextLevelExp: nextLevelExp.value,
      // 统计数据
      gameCount: gameCount.value,
      roomCount: roomCount.value
    }
    
    Taro.setStorageSync('userInfo', JSON.stringify(userData))
    return userData
  }
  
  // 从本地存储加载用户状态
  function loadUserFromStorage(): boolean {
    try {
      const userInfo = Taro.getStorageSync('userInfo')
      if (!userInfo) return false
      
      const data = JSON.parse(userInfo)
      openid.value = data.openid || ''
      nickname.value = data.nickname || ''
      avatar.value = data.avatar || ''
      role.value = data.role || UserRole.USER
      isRegistered.value = !!data.isRegistered
      currentRoom.value = data.currentRoom || null
      
      // 加载经验值系统数据
      points.value = data.points || 0
      exp.value = data.exp || 0
      level.value = data.level || 1
      nextLevelExp.value = data.nextLevelExp || 100
      
      // 加载统计数据
      gameCount.value = data.gameCount || 0
      roomCount.value = data.roomCount || 0
      
      return true
    } catch (error) {
      console.error('加载用户信息失败:', error)
      return false
    }
  }
  
  // 初始化用户信息
  async function initUser() {
    loading.value = true
    error.value = null
    
    try {
      // 1. 尝试从本地存储加载
      const localDataLoaded = loadUserFromStorage()
      
      // 2. 如果本地有数据且已注册，直接返回成功
      if (localDataLoaded && isRegistered.value) {
        return true
      }
      
      // 3. 如果本地有openid但未注册，保持现状
      if (localDataLoaded && openid.value && !isRegistered.value) {
        return true
      }
      
      // 4. 否则，进行静默登录获取openid
      return await silentLogin()
    } catch (e) {
      console.error('初始化用户失败:', e)
      error.value = '初始化用户失败'
      return false
    } finally {
      loading.value = false
    }
  }
  
  // 静默登录 - 仅获取openid
  async function silentLogin() {
    loading.value = true
    error.value = null
    
    try {
      const loginRes = await Taro.login()
      if (!loginRes.code) {
        error.value = '登录失败'
        return false
      }
      
      const result = await userService.getOpenId(loginRes.code)
      if (!result.success || !result.data) {
        error.value = result.message || '获取OpenID失败'
        return false
      }
      
      // 设置openid
      openid.value = result.data.openid
      saveUserToStorage()
      return true
    } catch (e) {
      console.error('静默登录失败:', e)
      error.value = '静默登录失败'
      return false
    } finally {
      loading.value = false
    }
  }
  
  // 注册用户 - 获取用户信息
  async function registerUser() {
    loading.value = true
    error.value = null
    
    try {
      // 确保有openid
      if (!openid.value) {
        const loginSuccess = await silentLogin()
        if (!loginSuccess) return false
      }
      
      // 微信环境使用getUserProfile
      if (process.env.TARO_ENV === 'weapp') {
        const profileRes = await Taro.getUserProfile({
          desc: '用于完善用户资料'
        })
        
        if (!profileRes.userInfo) {
          error.value = '获取用户信息失败'
          return false
        }
        
        // 更新本地状态
        nickname.value = profileRes.userInfo.nickName
        avatar.value = profileRes.userInfo.avatarUrl
        isRegistered.value = true
        
        // 保存到本地存储
        saveUserToStorage()
        
        // 同步到云端 - 使用Service层处理
        await userService.saveUserToCloud({
          openid: openid.value,
          nickname: nickname.value,
          avatar: avatar.value,
          role: role.value
        })
        
        return true
      } else {
        // 非微信环境模拟注册
        nickname.value = '模拟用户_' + Date.now().toString().slice(-4)
        avatar.value = 'https://example.com/avatar.jpg'
        isRegistered.value = true
        
        // 保存到本地
        saveUserToStorage()
        return true
      }
    } catch (e) {
      console.error('用户注册失败:', e)
      error.value = '用户注册失败'
      return false
    } finally {
      loading.value = false
    }
  }
  
  // 设置用户角色
  function setRole(newRole: UserRole) {
    role.value = newRole
    saveUserToStorage()
  }
  
  // 设置当前房间
  function setCurrentRoom(roomId: string) {
    currentRoom.value = roomId
    saveUserToStorage()
  }
  
  // 退出登录
  function logout() {
    openid.value = ''
    nickname.value = ''
    avatar.value = ''
    isRegistered.value = false
    currentRoom.value = null
    currentGameId.value = null
    
    // 清除本地存储
    Taro.removeStorageSync('userInfo')
  }
  
  // 设置当前游戏
  function setCurrentGame(gameId: string | null) {
    currentGameId.value = gameId
  }
  
  // 重置用户角色到普通用户
  function resetToNormalUser() {
    role.value = UserRole.USER
    saveUserToStorage()
  }
  
  // 清除房间和游戏信息
  function clearGameSession() {
    currentRoom.value = null
    currentGameId.value = null
    saveUserToStorage()
  }
  
  // 经验值系统相关方法
  
  // 增加积分
  function addPoints(amount: number) {
    points.value += amount
    saveUserToStorage()
  }
  
  // 增加经验值
  function addExp(amount: number) {
    exp.value += amount
    
    // 检查是否升级
    checkLevelUp()
    
    saveUserToStorage()
    return { level: level.value, exp: exp.value, nextLevelExp: nextLevelExp.value }
  }
  
  // 检查是否升级
  function checkLevelUp() {
    while (exp.value >= nextLevelExp.value) {
      // 升级
      level.value++
      exp.value -= nextLevelExp.value
      
      // 调整下一级所需经验值 (每级增加20%)
      nextLevelExp.value = Math.floor(nextLevelExp.value * 1.2)
    }
  }
  
  // 统计数据相关方法
  
  // 记录参与游戏
  function recordGameParticipation() {
    gameCount.value++
    saveUserToStorage()
  }
  
  // 记录创建房间
  function recordRoomCreation() {
    roomCount.value++
    saveUserToStorage()
  }
  
  return {
    // 状态
    openid,
    nickname,
    avatar,
    role,
    isRegistered,
    currentRoom,
    currentGameId,
    loading,
    error,
    userInfo,
    
    // 经验值系统
    points,
    exp,
    level,
    nextLevelExp,
    expPercentage,
    
    // 游戏统计
    gameCount,
    roomCount,
    
    // 方法
    initUser,
    registerUser,
    setRole,
    setCurrentRoom,
    logout,
    setCurrentGame,
    resetToNormalUser,  // 方法名称更新
    clearGameSession,
    
    // 经验值系统方法
    addPoints,
    addExp,
    
    // 统计方法
    recordGameParticipation,
    recordRoomCreation
  }
})